/**
  * ================================================================================
  * CompoundShape.java: A scene shape that is composed of multiple geometric shapes.
  * ================================================================================
  */

package shape;

import java.awt.*;
import java.awt.geom.*;
import java.awt.BasicStroke;
import java.awt.Color;

public abstract class CompoundShape extends SelectableShape {
   private Color    color = null;        // Color of compound shape ...
   private Color oldColor = Color.black; // Default color is black ...
   protected String sName = null;        // Name of compound shape ....
   protected double     x, y;  // Anchor point for compound shape ...
   protected double    width;  // Width  of compound shape ....
   protected double   height;  // Height of compound shape ....
   private GeneralPath  path;  // General path ...
   protected int textOffSetX =   0; // Default x- offset for text label.
   protected int textOffSetY = -10; // Default y- offset for text label.
   boolean filledShape = false;
   boolean shapeActive = false;
   boolean DEBUG = false;

   public CompoundShape() {
      path = new GeneralPath();
   }

   protected void add(Shape s) {
      path.append(s, false);
   }

   public boolean contains( Point2D pt ) {
      boolean insideX = ( pt.getX() > x ) && ( pt.getX() < (x + width));
      boolean insideY = ( pt.getY() < y ) && ( pt.getY() > (y - height));

      // Set shapeActive to true when cursor is inside shape ...

      if ( insideX && insideY == true )
         shapeActive = true;  
      else 
         shapeActive = false;  

      if (DEBUG == true) {
         System.out.println("*** In CompoundShape.contains(): name = " + sName );
         System.out.println("***                            : insideX = " + insideX );
         System.out.println("***                            : insideY = " + insideY );
         System.out.println("***                            : shapeActive = " + shapeActive );
      }

      return ( insideX && insideY );
   }

   public void translate(int dx, int dy) {
      path.transform( AffineTransform.getTranslateInstance(dx, dy) );
   }

   // Set/get name of compound shape ....

   public void setName ( String sName ) {
      this.sName = sName;
   }

   public String getName () {
      return sName;
   }

   // Set x- and y-offsets for text string ...

   public void setTextOffSetX( int textOffSetX ) {
      this.textOffSetX = textOffSetX;
   }

   public void setTextOffSetY( int textOffSetY ) {
      this.textOffSetY = textOffSetY;
   }

   // Set filled shape flag ....

   public void setColor( Color color ) {
      this.color = this.oldColor = color;
   }

   // Set filled shape flag ....

   public void setFilledShape ( boolean filledShape ) {
      this.filledShape = filledShape;
   }

   // Draw shape ....

   public void draw( Graphics2D g2D ) {
      g2D.translate(  x,  y );
      g2D.scale( 1.0, -1.0 );

      // Highlight shape when "cursor over shape" ....

      if (shapeActive == false)
          g2D.setColor( oldColor );
      else {
          g2D.setColor( Color.red );
      }

      // Set flag for filled shape ...

      if (filledShape == false)
          g2D.draw(path);
      else
          g2D.fill(path);

      if (shapeActive == false)
          g2D.setColor( oldColor );

      // Add label to shape ....

      if (sName != null)
          g2D.drawString( sName, textOffSetX, textOffSetY );

      // Reset affine transformation ....

      g2D.scale( 1.0, -1.0 );
      g2D.translate( -x, -y );
   }
}
